/**
 * @file
 * Javascript extensions for bilderblock Drupal theme.
 */

(function($) {

  Drupal.bilderblock = {

    // Remembers the last content scroll state while the overlay
    // menu is active, to work around some restrictions of the layout.
    scroll: false,

    /**
     * Sets a cookie.
     *
     * @param val
     */
    updateCookie: function(val) {
      $.cookie('Drupal.visitor.bilderblockMenu', val, {
        expires: 365,
        path: '/'
      });
    }, // Drupal.bilderblock.cookie
    /**
     * Returns the containing "body" element for the element applied to.
     *
     * @return object
     *   A jQuery result.
     */
    body: function() {
      return $(this).closest('body');
    }, // Drupal.bilderblock.body
    menu: {
      /**
       * Menu close switch event handler.
       *
       * Explicitely closes the menu. For use with swipe events etc.
       */
      close: function(event) {
        return Drupal.bilderblock.menu.toggle.call(this, event, 'closed');
      }, // Drupal.bilderblock.menu.close
      /**
       * Menu open switch event handler.
       *
       * Explicitely opens the menu. For use with swipe events etc.
       */
      open: function(event) {
        return Drupal.bilderblock.menu.toggle.call(this, event, 'open');
      }, // Drupal.bilderblock.menu.open
      /**
       * Returns the target state for toggling the current menu state.
       *
       * Checks the current menu state and returns the opposite state,
       * so the state and the cookie can be changed in sync.
       *
       * @return string
       *   - "closed", if the menu is closed.
       *   - "open" otherwise.
       */
      targetState: function() {
        return Drupal.bilderblock.body.apply(this)
          .hasClass('bilderblock-menu-closed') ? 'open' : 'closed';
      }, // Drupal.bilderblock.menu.targetState
      /**
       * Menu toggle switch event handler.
       *
       * Sets the "bilderblock-menu-closed" control class depending on either
       * the current menu state or an explicit targetState value.
       *
       * @param event
       * @param string targetState
       *   Optional forced target state:
       *   - 'closed'
       *   - 'open'
       *
       * @return object
       *   this
       */
      toggle: function(event, targetState) {
        event.preventDefault();
        var body = Drupal.bilderblock.body.apply(this);
        targetState = targetState || Drupal.bilderblock.menu.targetState.apply(this);
        body.toggleClass('bilderblock-menu-closed', targetState == 'closed');
        Drupal.bilderblock.updateCookie(targetState);
        return $(this);
      }, // Drupal.bilderblock.menu.toggle
    }, // Drupal.bilderblock.menu
    overlay: {
      /**
       * Determines whether JS menu overlay is currently displayed.
       *
       * @return bool
       */
      state: function() {
        return Drupal.bilderblock.body.apply(this)
          .hasClass('bilderblock-menu-overlay');
      }, // Drupal.bilderblock.overlay.state
      /**
       * Closes the overlay menu without suppressing the original event.
       *
       * Used to extend the "up" button, so it closes an open overlay menu.
       *
       * @return object
       *   this
       */
      closeUp: function() {
        var body = Drupal.bilderblock.body.apply(this);
        // Only update scroll marker if content is currently visible.
        if (body.hasClass('bilderblock-menu-overlay')) {
          body.removeClass('bilderblock-menu-overlay');
          // Conditionally re-scroll.
          if (Drupal.bilderblock.scroll) {
            $(window).scrollTop(Drupal.bilderblock.scroll);
          }
        }
        return $(this);
      }, // Drupal.bilderblock.overlay.toggle
      /**
       * Small-screen overlay menu toggle switch event handler.
       *
       * Toggles the "bilderblock-menu-overlay" control body class.
       *
       * @param event
       *
       * @return object
       *   this
       */
      toggle: function(event) {
        event.preventDefault();
        var body = Drupal.bilderblock.body.apply(this);
        // Only update scroll marker if content is currently visible.
        if (!body.hasClass('bilderblock-menu-overlay')) {
          Drupal.bilderblock.scroll = $(window).scrollTop();
        }
        body.toggleClass('bilderblock-menu-overlay');
        Drupal.bilderblock.overlay.tooltip.apply(this);
        // Only re-scroll if we are now open.
        if (!body.hasClass('bilderblock-menu-overlay') && Drupal.bilderblock.scroll) {
          $(window).scrollTop(Drupal.bilderblock.scroll);
        }
        return $(this);
      }, // Drupal.bilderblock.overlay.toggle
      /**
       * Creates a tooltip for menu toggle buttons, depending on menu state.
       *
       * @return string
       */
      tooltip: function() {
        var title = Drupal.bilderblock.overlay.state.apply(this) ?
          Drupal.t('Hide menu') : Drupal.t('Show menu');
        $(this).closest('body').find('div#menuswitch a.bb-top, div#menu-toggle-small a')
          .attr('title', title);
        return $(this);
      } // Drupal.bilderblock.overlay.tooltip
    } // Drupal.bilderblock.overlay
  }; // Drupal.bilderblock

  /**
   * Create our own namespace.
   */
  Drupal.behaviors.bilderblock = {
    attach: function(context) {
      // Bind menu swipe and double-click events.
      $('div#menu', context).once('bilderblock-menu-toggle', function() {
        $(this)
          // Inobtrusively connect jQuery mobile events.
          .bind('swiperight', Drupal.bilderblock.menu.close)
          .bind('swipeleft', Drupal.bilderblock.menu.open)
          // Default menu toggle.
          .dblclick(Drupal.bilderblock.menu.toggle);
      });
      // Bind menu toggle switches events.
      $('div#menuswitch a.bb-toggle', context).once('bilderblock-menu-toggle', function() {
        $(this).click(Drupal.bilderblock.menu.toggle);
      });
      // The "menu button" at the top simply toggles the JS menu.
      $('div#menu-toggle-small a', context).once('bilderblock-overlay-toggle', function() {
        $(this).click(Drupal.bilderblock.overlay.toggle);
        Drupal.bilderblock.overlay.tooltip.apply(this);
      });
      // The "up" botton eventually hides the JS menu and then jumps to the page top.
      $('div#menuswitch a.bb-top', context).once('bilderblock-overlay-toggle', function() {
        $(this).click(Drupal.bilderblock.overlay.closeUp);
      });
      // Initially hide menu.
      Drupal.bilderblock.body.apply($('div#menu', context)).addClass('bilderblock-menu-has-overlay');
      // Update the cookie with the current value,
      // at least once per page request to reset expiration.
      // (Only if a cookie exists.)
      var c = $.cookie('Drupal.visitor.bilderblockMenu');
      if (c) {
        Drupal.bilderblock.updateCookie(c);
      }
    }
  };

})(jQuery);
